﻿Type=Class
Version=5.02
ModulesStructureVersion=1
B4A=true
@EndOfDesignText@
'Class module
Sub Class_Globals
	Private TmrScroll	As Timer
	Private Index		As Long
	Private Target		As Label			'the label on wich operate
	Private Module		As Object			'the calling module, for events
	Private Event		As String			'the event name in your module
	Private Txt			As String			'the scrolling text
	Private TxtBackup	As String			'the backup of the string, only the window size
	Private WinSize		As Long				'text window size
	Private Scrolls		As Long				'number of scrolls
	Private scrollLoop	As Boolean			'true if continuous scrolling
	Private FromLtoR	As Boolean			'true if scroll from left to right
	Private PingPong    As Boolean			'true if scroll from side to side (like ping pong)
	
End Sub

'Initializes the object.
'Pass the module and the view on wich you want the scrolling text and the scrolling ended event name.
'The event is fired ONLY when the 'Repeat' parameter in ScrollLeftRight or ScrollRightLeft is greater than 0 and the scrolling has ended
'Pass "" if no event is required
'The called event suffix must be '_scrollended'
'Note that you can pass labels, edittexts, buttons
Public Sub Initialize(TargetModule As Object,TargetLabel As Label,TargetEvent As String)
	Target=TargetLabel
	Module=TargetModule
	Event =TargetEvent	
	TmrScroll.Initialize("tmrscroll",500)

End Sub

'Start the scrolling text from Left to Right.
'Pass the text as string, the text window size in n. of chrs,the scroll speed and the number of scrolls.
'Repeat: pass 0 if you want continuous scroll else pass the number of scrolls you want.
'Text: pass "" if your target already has a text and you wish to use it.
'Window: the number of chrs you want in your label, must be >0
Public Sub ScrollLeftRight(Text As String,Window As Long, Speed As Long,Repeat As Long)
	If Window<=0 Then Return
	TmrScroll.Enabled=False
	Index=0
	WinSize=Window
	Scrolls=Repeat
	Txt=Text
	FromLtoR=True									'direction
	PingPong=False									'no ping pong effect
	
	'get the text from the target if needed
	If Txt="" Then Txt=Target.Text
	
	'set the text for end of scroll when Repeat>0
	TxtBackup=Txt.SubString(Min(WinSize,Txt.Length))
	
	'add initial spaces for the same window size for nice effect...
	Dim St1 As String
	For i = 1 To WinSize
		St1=St1 & " "
	Next
	Txt=Txt & St1
	
	'set if looping
	If Scrolls<=0 Then scrollLoop=True Else scrollLoop=False	
	'the initial target text depends on the direction
	Target.Text=Txt.SubString(Min(WinSize,Txt.Length))
	
	'start scroll
	TmrScroll.Interval=Speed
	TmrScroll.Enabled=True
	
End Sub

'Start the scrolling text from Right to Left
'Pass the text as string, the text window size in n. of chrs,the scroll speed and the number of scrolls.
'Repeat: pass 0 if you want continuous scroll else pass the number of scrolls you want.
'Text: pass "" if your target already has a text and you wish to use it.
'Window: the number of chrs you want in your label, must be >0
Public Sub ScrollRightLeft(Text As String,Window As Long, Speed As Long,Repeat As Long)
	If Window<=0 Then Return
	TmrScroll.Enabled=False
	Index=0
	WinSize=Window
	Scrolls=Repeat
	Txt=Text
	FromLtoR=False									'direction
	PingPong=False									'no ping pong effect
	
	'get the text from the target if needed
	If Txt="" Then Txt=Target.Text
	
	'set the text for end of scroll when Repeat>0
	TxtBackup=Txt.SubString2(0,Min(WinSize,Txt.Length))
	
	'add initial spaces for the same window size for nice effect...
	Dim St1 As String
	For i = 1 To WinSize
		St1=St1 & " "
	Next
	Txt=St1 & Txt
	
	'set if looping
	If Scrolls<=0 Then scrollLoop=True Else scrollLoop=False	
	'the initial target text depends on the direction
	Target.Text=Txt.SubString2(0,Min(WinSize,Txt.Length))
	
	'start scroll
	TmrScroll.Interval=Speed
	TmrScroll.Enabled=True
	
End Sub

'Start the scrolling with an effect like ping pong.
'Pass the text as string, the text window size in n. of chrs,the scroll speed and the number of scrolls.
'Repeat: pass 0 if you want continuous scroll else pass the number of scrolls you want.
'Text: pass "" if your target already has a text and you wish to use it.
'Window: the number of chrs you want in your label, must be >0
public Sub ScrollPingPong(Text As String,Window As Long, Speed As Long,Repeat As Long)
	If Window<=0 Then Return
	TmrScroll.Enabled=False
	Index=0
	WinSize=Window
	Scrolls=Repeat
	Txt=Text
	FromLtoR=False									'direction
	PingPong=True									'ping pong effect active
	
	'get the text from the target if needed
	If Txt="" Then Txt=Target.Text
	'set the text for end of scroll when Repeat>0
	TxtBackup=Txt.SubString2(0,Min(WinSize,Txt.Length))
	
	'add spaces before and after the string for the same window size for nice effect...
	Dim St1 As String
	For i = 1 To WinSize
		St1=St1 & " "
	Next
	Txt=St1 & Txt				'effect 1
	'Txt=" " & Txt & " "		'effect 2
	'set if looping
	If Scrolls<=0 Then scrollLoop=True Else scrollLoop=False	
	'start scroll
	TmrScroll.Interval=Speed
	TmrScroll.Enabled=True

End Sub

'Check if the text is scrolling.
'Return true if is scrolling
'Return false if not scrolling
public Sub IsScrolling As Boolean
	Return TmrScroll.Enabled
End Sub

'Stop the scrolling text.
'You should call Stop when your app is paused.
public Sub Stop
	TmrScroll.Enabled=False
End Sub

'start the scrolling text without changes.
'you should call Start when your app is resumed
public Sub Start
	TmrScroll.Enabled=True
End Sub

'Get or set the scrolling speed in milliseconds (same as a timer)
public Sub setSpeed(Speed As Long)
	TmrScroll.Enabled=False
	TmrScroll.Interval=Speed
	TmrScroll.Enabled=True
End Sub

'Get or set the scrolling speed in milliseconds (same as a timer)
public Sub getSpeed As Long
	Return TmrScroll.Interval
End Sub

'scrolling text engine
Private Sub tmrscroll_tick
	If Txt.Length=0 Then Return							'avoid errors
	Dim St1 As String
	Dim St2 As String
	
	If PingPong Then
		'pingpong mode...
		'check for scroll end
		If Not(scrollLoop) And Scrolls=0 Then	
			'set the text for end of scrolls and stop scroll
			TmrScroll.Enabled=False
			Target.Text=TxtBackup
			'call the handler if exist
			If SubExists(Module, Event & "_scrollended") Then
				CallSub(Module, Event & "_scrollended")
			End If	
		Else
			'scroll	
			If Not(FromLtoR) Then
				'scroll from right to left, the first chr appear first
				St1= Txt.SubString(1)
				St2= Txt.CharAt(0)					'get first chr
			Else
				'scroll from left to right, the last chr appear first
				St2= Txt.SubString2(0,Txt.Length-1)
				St1= Txt.CharAt(Txt.Length-1)		'get last chr
			End If
			
			Txt=St1 & St2
			Target.Text=Txt.SubString2(0,Min(WinSize,Txt.Length))
			Index=(Index+1) mod Txt.Length			'keep track of current position
			If Index=0 Then FromLtoR=Not(FromLtoR)
			If Not(scrollLoop) And Index=0 Then Scrolls=Scrolls-1
		End If
	Else
		'scroll mode...
		'check for scroll end
		If Not(scrollLoop) And Scrolls=0 Then	
			'set the text for end of scrolls and stop scroll
			TmrScroll.Enabled=False
			Target.Text=TxtBackup
			'call the handler if exist
			If SubExists(Module, Event & "_scrollended") Then
				CallSub(Module, Event & "_scrollended")
			End If	
		Else
			'scroll	
			If Not(FromLtoR) Then
				'scroll from right to left, the first chr appear first
				St1= Txt.SubString(1)
				St2= Txt.CharAt(0)					'get first chr
			Else
				'scroll from left to right, the last chr appear first
				St2= Txt.SubString2(0,Txt.Length-1)
				St1= Txt.CharAt(Txt.Length-1)		'get last chr
			End If
			
			Txt=St1 & St2
			Target.Text=Txt.SubString2(0,Min(WinSize,Txt.Length))
			Index=(Index+1) mod Txt.Length			'keep track of current position
			If Not(scrollLoop) And Index=0 Then Scrolls=Scrolls-1
		End If
	End If
	
End Sub